{#- Template for TwinCAT PLC Projects.

This template renders a TwinCAT PLC Project.

Context:
  data: The PLC Project object to render.
  root (bool): Whether this is the root object, injected with `:::` in a Markdown page.
  heading_level (int): The HTML heading level to use.
  config (dict): The configuration options.
-#}

{% extends "_base/tcpou.html.jinja" %}

{% block title %}{{ data.name }} - TwinCAT PLC Project Documentation{% endblock %}

{% block labels scoped %}
  {#- Labels block.

  This block renders the labels for the PLC Project.
  -#}
  <span class="doc doc-label doc-label-plcproject">PLC Project</span>
{% endblock labels %}
